<?php
/**
 * Created by PhpStorm.
 * User: matheussilva
 * Date: 30/10/14
 * Time: 11:45
 */

namespace Folha\Logic\ImportacaoLoteEvento;

use \Folha\Entities\FpaItemlotelanc;

use \Folha\Entities\FpaLotelancevento;

use \Folha\Entities\FpaEventoh;

use \Folha\Entities\GpeVinculom;

use \Folha\Entities\FpaEventom;

use \Geral\Util\Ambiente;

use \HCM\Util\DateUtil;

use \Folha\Repositories\FpaLotelancevento as Lotelancevento;

use \Folha\Repositories\FpaItemlotelanc as Itemlotelanc;


class Importacao
{

    const XLS = 1;
    const XLSX = 2;

    
    /**
     * CSU002 - Gerenciador de Importação
     *
     * @param String $caminho
     * @param int $nrorg
     * @param String $dtmescompetencia
     *
     * @throws \Exception
     */
    public function __construct( $caminho, $nrorg, $dtmescompetencia)
    {
        try {
            $this->validaExtensao($caminho);

            $info = new ManipulacaoExcel();
            $info = $info->perparaInformacao($caminho);

            $this->validaFormatacao($info);
            $this->validaInformacao($info, $nrorg, $dtmescompetencia);
            $infoFinal = $this->preparaInfo($caminho, $info, $nrorg, $dtmescompetencia);
            $this->validaConsistencia($info);
            $this->insereLoteEvento($infoFinal['LOTE']);
            $this->insereItemEvento($infoFinal['ITEM']);
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }

    }

    /**
     * CSU003 - Validador de Extensão de Arquivo.
     *
     * @param $caminho
     *
     * @return bool
     * @throws \Exception
     */
    private function validaExtensao( $caminho)
    {
        try {
            $nome = preg_split("/(\\|\/|\.)/", $caminho);
            $extencao = strtoupper(end($nome));
            switch ($extencao) {
                case('XLS'):{
                    return self::XLS;
                }
                case('XLSX'):{
                    return self::XLSX;
                }
                default:{
                    throw Exception::extensaoInvalida();
                }
            }
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU004 - Validador de Formato do Arquivo
     *
     * @param array $info
     *
     * @return bool
     * @throws \Exception
     */
    private function validaFormatacao( $info)
    {
        try {
            $this->validaFormatacaoFuncionario($info['FUNCIONARIO']);
            $this->validaFormatacaoEvento($info['EVENTO']);
            $this->validaFormatacaoInfos($info['INFO']);
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU004a - Valida o formato dos funcionarios
     *
     * @param array $funcs
     *
     * @return bool
     * @throws \Exception
     */
    private function validaFormatacaoFuncionario( $funcs)
    {
        try {
            if ($funcs) {
                foreach ($funcs as $func) {
                    if (intval($func['CODIGO']) == 0) {
                        throw Exception::funcionarioInvalido($func['FUNCIONARIO']);
                    }
                }
                return true;
            } else {
                throw Exception::semVinculos();
            }
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU004b - Valida o formato dos eventos
     *
     * @param $eventos
     *
     * @return bool
     * @throws \Exception
     */
    private function validaFormatacaoEvento( $eventos)
    {
        try {
            if ($eventos) {
                foreach ($eventos as $evento) {
                    if (intval($evento['CODIGO']) == 0) {
                        throw Exception::eventoInvalido($evento['EVENTO']);
                    }
                }
            } else {
                throw Exception::semEventos();
            }

            return true;
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU004c - Valida o formato das informações que serão persistidas
     * @param $infos
     *
     * @return bool
     * @throws \Exception
     */
    private function validaFormatacaoInfos( $infos)
    {
        try {
            foreach ($infos as $info) {
                if(intval($info['CODIGO']) == 0){
                    throw Exception::funcionarioInvalido($info['FUNCIONARIO']);
                }
                if(intval($info['EVENTO']) == 0){
                    throw Exception::eventoInvalido($info['EVENTO']);
                }
                if(!($info['DTMESCOMPETENCIA'] instanceof \DateTime)){
                    throw Exception::dataCompetenciaInvalido();
                }
                if(($info['VALOR'] != 0)&&(floatval($info['VALOR']) < 0)&&(!is_null($info['VALOR']))){
                    throw Exception::valorInvalido($info['EVENTO'], $info['CODIGO']);
                }
            }
            return true;
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005 - Validador das Informações do Arquivo.
     *
     * @param array $info
     * @param int $nrorg
     * @param String $dtmescompetencia
     *
     * @return bool
     * @throws \Exception
     */
    private function validaInformacao( $info, $nrorg, $dtmescompetencia)
    {
        try {
            $this->validaInformacaoFuncionario($info['FUNCIONARIO'], $nrorg);
            $this->validaInformacaoEvento($info['EVENTO'], $nrorg);
            $this->validaInformacaoInfo($info['INFO'], $nrorg, $dtmescompetencia);
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005a - Valida as informações do funcionário
     *
     * @param $funcs
     * @param $nrorg
     *
     * @return bool
     * @throws \Exception
     */
    private function validaInformacaoFuncionario( $funcs, $nrorg)
    {
        try {
            foreach ($funcs as $func) {

                $params = array(
                    'P_NRVINCULOM' => $func['CODIGO'],
                    'P_NRORG' => $nrorg
                );
                $where = "NRORG = :P_NRORG AND NRVINCULOM = :P_NRVINCULOM";
                $vinculo = GpeVinculom::getObjectByWhere($where, $params);

                if (!$vinculo){
                    throw Exception::funcionarioInexistente($func['FUNCIONARIO']);
                }
            }
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005b - Valida informações do Evento
     *
     * @param $eventos
     * @param $nrorg
     *
     * @return bool
     * @throws \Exception
     */
    private function validaInformacaoEvento( $eventos, $nrorg)
    {
        try {
            foreach ($eventos as $evento) {

                $params = array(
                    'P_NREVENTOM' => $evento['CODIGO'],
                    'P_NRORGPADRAO' => 0,
                    'P_NRORG' => $nrorg
                );
                $where = "(NRORG = :P_NRORG OR NRORG = :P_NRORGPADRAO) AND NREVENTOM = :P_NREVENTOM";
                $objeto = FpaEventom::getObjectByWhere($where, $params);

                if (!$objeto) {
                    throw Exception::eventoInvalido($evento['EVENTO']);
                }
            }
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU005c - Valida informação que será persistida
     *
     * @param array $infos
     * @param int $nrorg
     * @param String $dtmescompetencia
     *
     * @return bool
     * @throws \Exception
     */
    private function validaInformacaoInfo( $infos, $nrorg, $dtmescompetencia)
    {
        try {
            $competencia = substr($dtmescompetencia, 0, 10);
            if ($competencia != $infos[0]['DTMESCOMPETENCIA']->format('d/m/Y')) {
                throw Exception::dataCompetenciaIncorreta();
            }
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }


    /**
     * CSU007 - Verificação de consistência do arquivo inserido.
     *
     * @param array $info
     *
     * @return bool
     * @throws \Exception
     */
    private function validaConsistencia( $info)
    {
        try {
            return true;
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU008 - Inserção de Lotes de Eventos
     *
     * @param array $lote
     *
     * @return bool
     * @throws \Exception
     */
    private function insereLoteEvento( $lote)
    {
        Lotelancevento::insereItem($lote);
    }

    /**
     * CSU009 - Inserção de Itens de lotes de eventos.
     *
     * @param array $itens
     *
     * @return bool
     * @throws \Exception
     */
    private function insereItemEvento( $itens)
    {
        foreach ($itens as $item) {
            Itemlotelanc::insereItem($item);
        }
    }

    /**
     * CSU010 - Prepara a informação para receber a última validação e ser persistida.
     *
     * @param $caminho
     * @param $info
     * @param $nrorg
     * @param $dtmescompetencia
     *
     * @return array
     * @throws \Exception
     */
    private function preparaInfo( $caminho, $info, $nrorg, $dtmescompetencia)
    {
        try {
            /**
             * @var $lotevento FpaLotelancevento
             */

            $lotevento = $this->preparaInfoLoteEvento($caminho, $nrorg, $dtmescompetencia);
            $itensEvento = $this->preparaInfoItemEvento($info['INFO'], $nrorg, $lotevento->getNrlotelancevento());

            return array(
                'LOTE' => $lotevento,
                'ITEM' => $itensEvento
            );
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }


    /**
     * CSU010a - Prepara a informação de Lote de Eventos
     *
     * @param $caminho
     * @param $nrorg
     * @param $dtmescompetencia
     *
     * @return FpaLotelancevento
     * @throws \Exception
     */
    private function preparaInfoLoteEvento( $caminho, $nrorg, $dtmescompetencia)
    {
        /**
         * @var FpaLotelancevento $loteevento
         */
        try {
            $nome = $this->preparaInfoNome($caminho);
            $nrlotelacevento = $this->novoCodigo('FPA_LOTELANCEVENTO', $nrorg);

            $loteevento = new FpaLotelancevento();

            $loteevento->setNrlotelancevento($nrlotelacevento);
            $loteevento->setDslotelancevento("IMPORTAÇÃO DE ARQUIVO $nome");
            $loteevento->setDtlotelancevento(DateUtil::getDataAtual());
            $loteevento->setDtcompetloteeve(DateUtil::getDataDeString($dtmescompetencia));
            $loteevento->setIdstatuslotelanc('PENDENTE');
            $loteevento->setDsobslotelanc('');
            $loteevento->setNrtipomoviment(1);
            $loteevento->setIdoriglotelanc('PLANILHA');
            $loteevento->setIdativo('S');
            $loteevento->setNrorginclusao($nrorg);
            $loteevento->setDtinclusao(DateUtil::getDataAtual());
            $loteevento->setCdoperinclusao(Ambiente::getCdOperador());
            $loteevento->setNrorgultatu($nrorg);
            $loteevento->setDtultatu(DateUtil::getDataAtual());
            $loteevento->setCdoperultatu(Ambiente::getCdOperador());
            $loteevento->setNrorg($nrorg);

            return $loteevento;
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * CSU010b - Prepara as informações de itens de Eventos
     *
     * @param array $infos
     * @param int $nrorg
     * @param int $nrlotelacevento
     *
     * @return array
     * @throws \Exception
     *
     */
    private function preparaInfoItemEvento( $infos, $nrorg, $nrlotelacevento)
    {
        /**
         * @var FpaItemlotelanc $info
         * @var FpaEventoh $ref
         */
        try {
            $itens = array();
            foreach ($infos as $info) {

                if (!is_null($info['VALOR']) && !($info['VALOR'] == 0)) {
                    $codigo = $this->novoCodigo('FPA_ITEMLOTELANC', $nrorg);
                    $item = new FpaItemlotelanc();

                    $item->setNritlotelanc($codigo);
                    $item->setNrlotelancevento($nrlotelacevento);
                    $item->setNrvinculom($info['CODIGO']);
                    $item->setNreventom($info['EVENTO']);
                    $item->setIdativo('S');
                    $item->setNrorginclusao($nrorg);
                    $item->setDtinclusao(DateUtil::getDataAtual(false));
                    $item->setCdoperinclusao(Ambiente::getCdOperador());
                    $item->setNrorgultatu($nrorg);
                    $item->setDtultatu(DateUtil::getDataAtual(false));
                    $item->setCdoperultatu(Ambiente::getCdOperador());
                    $item->setNrorg($nrorg);

                    $params = array('P_NRORG' => Ambiente::getNrorgTrabalho(),
                        'P_DTMESCOMPETENCIA' => Ambiente::getDtmescompetencia(),
                        'P_NREVENTOM' => $info['EVENTO']);
                    $where = "(NRORG = :P_NRORG OR NRORG = 0) AND NREVENTOH IN (SELECT NREVENTOH FROM FPA_EVENTOH WHERE DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM FPA_EVENTOH MAX_EVENTOH WHERE MAX_EVENTOH.NRORG = FPA_EVENTOH.NRORG AND MAX_EVENTOH.NREVENTOM = FPA_EVENTOH.NREVENTOM AND MAX_EVENTOH.NREVENTOM = :P_NREVENTOM AND TRUNC(MAX_EVENTOH.DTMESCOMPETENC, 'DD') <= :P_DTMESCOMPETENCIA AND (MAX_EVENTOH.NRORG = :P_NRORG OR MAX_EVENTOH.NRORG = 0)))";
                    $ref = FpaEventoh::getObjectByWhere($where, $params);

                    if ($ref) {
                        if ($ref->getIdmediavalref() == 'REFERENCIA') {
                            $item->setVrreffolhalote($info['VALOR']);
                            $item->setVreventolote('');
                        } else {
                            $item->setVreventolote($info['VALOR']);
                            $item->setVrreffolhalote('');
                        }
                    }

                    $itens[] = $item;
                }

            }

            return $itens;
        } catch (\Exception $e) {
            if($e->getCode() == Exception::AVISO){
                tk_message($e->getMessage(), 'error');
            }else{
                throw $e;
            }
        }
    }

    /**
     * Retorna o nome do Arquivo recebido
     *
     * @param $caminho
     *
     * @return String
     */
    private function preparaInfoNome( $caminho)
    {
//        $nome = explode(PATH_SEPARATOR, $caminho);
        $nome = tk_get_uploaded_file_name();
        return $nome;
    }

    /**
     * Retorna o próximo código único para persistência.
     * @param $tabela
     * @param $nrorg
     * @return int
     */
    private function novoCodigo($tabela, $nrorg)
    {
        return \Geral\Util\NovoCodigo::geraCodigo($nrorg, $tabela, 1, 21, true);
    }




}